<?php

// query.php - Jukebox's main playback script

// Last updated 2009-10-03 for release 0.4.2.0

define("PLAYLIST_MAX", 100);

define("TRACK_FOLDER", "/tracks");  // The subfolder where tracks are perminately stored after being processed.

require_once "include/database.php";
require_once "include/settings.php";
require_once "include/common.php";
require_once "include/class.rc4crypt.php";

dbconn();


function check_key($password, $crypt_b64) {
	$time_margin = 3600;
	if(!$crypt_b64) {
		print("DEBUG: Time = " . time() . ", key not provided.<BR />\n");
		return false;
	}
	$decrypt = unpack("l1", rc4crypt::decrypt($password, pack("H*", $crypt_b64)));
	$decrypt = $decrypt[1];
	if (!is_numeric($decrypt)) {
		print("DEBUG: Time = " . time() . ", decrypted string = " . $decrypt . " - non-numeric<BR />\n");
		return false;
	}
	if(abs(time() - $decrypt) <= $time_margin) {
		return true;
	} else {
		print("DEBUG: Time = " . time() . ", decrypted string = " . $decrypt . " - bad match, difference = " . (time() - $decrypt) . "<BR />\n");
		return false;
	}
}


function hexbin($str_hex) {
  $str_bin = FALSE;
  for ($i=0; $i < strlen($str_hex); $i++) {
    $str_bin .= sprintf("%04s", decbin(hexdec($str_hex[$i])));
  }
  return $str_bin;
}


// Generate MySQL audio query
function audio_query() {
	global $audio_tables, $remote_server_on, $leech_pass;
	
	if(!check_key($leech_pass, get_or_null("key"))) {
		die("Leech protection: Credentials rejected.</BR>\n");
	}
	
	$id_list = get_or_null("id_list");
	$id_base64 = get_or_null("id_b64");
	$stream_id = get_or_null("sid");
	$id = get_or_null("id");
	
	if($stream_id) {
		$query = sprintf("SELECT pl.track_id FROM %s AS pl INNER JOIN %s AS st USING (stream_id) WHERE st.stream_id = %d AND pl.sequence >= st.now_playing ORDER BY pl.sequence", $audio_tables['playlists'], $audio_tables['streams'], mysql_clean_input($stream_id));
		$result = mysql_query($query) or sqlerrorhandler("(".mysql_errno().") ".mysql_error(), $query, $_SERVER['PHP_SELF'], __LINE__);
		$id_array = array();
		while($row = mysql_fetch_row($result)) {
			$id_array[] = $row[0];
		}
		$id_list = implode(",", $id_array);
	} elseif($id_list) {
		$id_list = mysql_clean_input($id_list);
	}	elseif ($id_base64) {
		$id_temp = base64url_decode($id_base64);
		$id_array = unpack("n*", $id_temp);
		$id_list = implode(",", $id_array);
		// print("Base64 hash = " . $id_base64 . ", id temp = " . $id_temp . ", id list = " . $id_list . "<BR />\n");// DEBUG
	} elseif ($id) {
		$id_list = mysql_clean_input($id);
	}
	
	if(!$id_list) {
		die("Error: No track ID numbers were provided.<BR />\n");
	}
	
	/*
	$query = sprintf("SELECT id, HEX(md5) AS md5, playtime, title, album, artist%s FROM %s%s WHERE id IN (%s) ORDER BY FIELD(id,%s) LIMIT %s",
			($remote_server_on) ? (", uploaded") : (""),
			$audio_tables['tracks'],
			($remote_server_on) ? (" LEFT JOIN " . $audio_tables['remote_server'] . " ON id = track_id") : (""),
			$id_list,
			$id_list,
			PLAYLIST_MAX
		);
	*/
	$query = sprintf("SELECT id, md5, uploaded FROM %s WHERE id IN (%s) ORDER BY FIELD(id,%s) LIMIT %s",
			$audio_tables['tracks'],
			$id_list,
			$id_list,
			PLAYLIST_MAX
		);
	
	// Generate query results
	$result = array();
	$temp = mysql_query($query) or sqlerrorhandler("(".mysql_errno().") ".mysql_error(), $query, $_SERVER['PHP_SELF'], __LINE__);
	while($row = mysql_fetch_array($temp)) {
		$result[] = $row;
	}
	
	return $result;
}



function build_playlist($query) {

	// global $query_types, $query_groups, $folder_depth, $root_url, $root_folder, $remote_server_on, $file_server_url, $audio_tables;
	global $query_types, $query_groups, $folder_depth, $remote_server_on, $file_server_url, $audio_tables;

	define("PLAYLIST_DIR", "playlists/");
	define("TRACK_DIR", "tracks/");
	$local_tracks = "http://" . $_SERVER['SERVER_NAME'] . dirname($_SERVER['SCRIPT_NAME']) . "/" . TRACK_DIR;
	$write_chmod = 0755;
	if($remote_server_on && substr($file_server_url, -1) != "/") {
		$file_server_url .= "/";
	}

	// $playlist_entry = "#EXTM3U\r\n";
	foreach($query as $target) {
		if($remote_server_on && $target['uploaded']) {
			$track_address = $file_server_url;
		} else {
			$track_address = $local_tracks;
		}
		$md5_hash = strtolower($target['md5']);
		for($i = 0; $i < $folder_depth; $i++) {
			$track_address .= $md5_hash[$i] . "/";
		}
		$track_address .= $md5_hash . ".mp3";
		// $playlist_entry .= "\r\n#EXTINF:".$target['playtime'].",".$target['title']." - ".$target['album']." - ".$target['artist']."\r\n";
		$playlist_entry .= $track_address."\r\n";
	}
	$md5_playlist = md5($playlist_entry);
	$file_playlist = PLAYLIST_DIR . $md5_playlist . ".m3u";
	if(!file_exists($file_playlist)) {
		if(!is_dir(PLAYLIST_DIR)) {
			if(!mkdir(PLAYLIST_DIR, $write_chmod, TRUE)) {
				die("Playlist creation failed!  Folder path = $path_playlist, file path = $file_playlist");
			}
		}
		$file_handle = fopen($file_playlist, w);
		fwrite($file_handle, $playlist_entry);
		fclose($file_handle);
		chmod($file_playlist, $write_chmod);
	}

	return $file_playlist;
}



function music_time($path_playlist, $volume = 80, $position = 0.0) {
	if($volume > 100) {
		$volume = 100;
	} elseif ($volume < 0) {
		$volume = 0;
	}
	$mp_volume = intval(-3000*((100 - $volume)/100));
	print <<<END
<HTML>
<HEAD>
	<TITLE>Jukebox - Source Engine Music System</TITLE>
</HEAD>
<BODY>
	<OBJECT ID="MediaPlayer1" CLASSID="CLSID:22d6f312-b0f6-11d0-94ab-0080c74c7e95" CODEBASE="http://activex.microsoft.com/activex/controls/mplayer/en/nsmp2inf.cab# Version=5,1,52,701" STANDBY="Loading Microsoft Windows Media Player components..." TYPE="application/x-oleobject" width="280" height="46">
		<PARAM NAME="fileName" VALUE="$path_playlist" />
		<PARAM NAME="animationatStart" VALUE="true" />
		<PARAM NAME="transparentatStart" VALUE="true" />
		<PARAM NAME="autoStart" VALUE="true" />
		<PARAM NAME="showControls" VALUE="true" />
		<PARAM NAME="Volume" VALUE="$mp_volume" />
		<PARAM NAME="CurrentPosition" VALUE="$position" />
		<EMBED TYPE="application/x-mplayer2" PLUGINSPAGE="http://www.microsoft.com/Windows/MediaPlayer/" SRC="$path_playlist" NAME="MediaPlayer1" WIDTH=280 HEIGHT=46 AUTOSTART=1 SHOWCONTROLS=1 CURRENTPOSITION=$position VOLUME=$mp_volume />
	</OBJECT>
</BODY>
</HTML>
END;

	return TRUE;
}



// The main control sequence

$query = audio_query();

$playlist = build_playlist($query);

$stream_id = get_or_null("sid");
$volume = choose_data_source(get_or_null("vol"), 80);
if($stream_id !== NULL) {
	$query = sprintf("SELECT st.volume_shift, UNIX_TIMESTAMP() - UNIX_TIMESTAMP(st.start_time) - COALESCE(SUM(tr.playtime), 0) AS track_position FROM %s AS st LEFT JOIN %s AS pl USING (stream_id) LEFT JOIN %s AS tr ON (pl.track_id = tr.id AND pl.sequence < st.now_playing) WHERE st.stream_id = %d GROUP BY st.stream_id", $audio_tables['streams'], $audio_tables['playlists'], $audio_tables['tracks'], $stream_id);
	$result = mysql_query($query) or sqlerrorhandler("(".mysql_errno().") ".mysql_error(), $query, $_SERVER['PHP_SELF'], __LINE__);
	$row = mysql_fetch_row($result);
	$volume += $row[0];
	$position = floatval($row[1]);
	if($position < 1.0) {
		$position = 0.0;
	}
} else {
	$position = choose_data_source(get_or_null("pos"), 0.0);
	$position = floatval($position);
}

music_time($playlist, $volume, $position);
?>
